@@ -9,9 +9,9 @@
      /_/   \_\___/  |_| \___/ |_| |_| \_\/_/   \_\____/|_____|_| \_\
     
 ---------------------------------------------------------------------------
-Kieran Mackle
-version 0.0.1
-
+     A Python-Based Development Platform For Automated Trading Systems
+                             Kieran Mackle
+                             Version 0.0.12
 """
 
 from getopt import getopt
@@ -29,13 +29,14 @@ from autotrader.emailing import emailing
 from autotrader.brokers.oanda import Oanda
 from autotrader.brokers.virtual.virtual_broker import Broker
 from autotrader.lib import logger, instrument_list, environment_manager, autodata
-from autotrader.autoplot import plot_backtest
+from autotrader import autoplot
 
 
 class AutoTrader():
     
     def __init__(self):
         self.config_file    = None
+        self.custom_config  = None
         self.verbosity      = 0
         self.show_help      = None
         self.notify         = 0
@@ -48,8 +49,20 @@ class AutoTrader():
         self.data_file      = None
         self.instruments    = None
         self.home_dir       = None
+        self.validation_file = None
+        self.plot_validation_balance = True
     
     def run(self):
+        if self.show_help is not None:
+            self.print_help(self.show_help)
+            return
+        
+        if self.config_file is None and self.backtest is False:
+            self.print_usage()
+        else:
+            self.main()
+    
+    def main(self):
         ''' -------------------------------------------------------------- '''
         '''                         Load configuration                     '''
         ''' -------------------------------------------------------------- '''
@@ -66,6 +79,10 @@ class AutoTrader():
             config_file_path    = os.path.join(home_dir, 'config', config_file)
             config              = self.read_yaml(config_file_path + '.yaml')
         
+        if self.validation_file is not None:
+            livetrade_history   = pd.read_csv(self.validation_file, index_col = 0)
+            livetrade_history   = livetrade_history.fillna(method='ffill')
+        
         # Read configuration file
         interval            = config["STRATEGY"]["INTERVAL"]
         period              = config["STRATEGY"]["PERIOD"]
@@ -91,6 +108,8 @@ class AutoTrader():
             scan_results    = {}
         elif self.instruments is not None:
             watchlist       = self.instruments.split(',') 
+        elif self.validation_file is not None:
+            raw_watchlist   = livetrade_history.Instrument.unique() # FOR OANDA
         else:
             watchlist       = config["WATCHLIST"]
         
@@ -121,9 +140,19 @@ class AutoTrader():
             if int(self.verbosity) > 0:
                 banner = pyfiglet.figlet_format("AutoBacktest")
                 print(banner)
+            
+            if self.validation_file is not None:
+                # Also get broker-specific utility functions
+                # TODO generalise per broker used
+                broker_utils = importlib.import_module('autotrader.brokers.oanda.utils') # FOR OANDA ONLY
                 
+                # Correct watchlist
+                if self.instruments is None:
+                    watchlist   = broker_utils.format_watchlist(raw_watchlist)
+            
         else:
-            utils           = importlib.import_module('autotrader.brokers.oanda.utils')
+            # TODO generalise per broker used
+            utils           = importlib.import_module('autotrader.brokers.oanda.utils') # FOR OANDA ONLY
             broker          = Oanda.Oanda(broker_config)
         
         if int(self.notify) > 0:
@@ -161,6 +190,21 @@ class AutoTrader():
             # Get price history
             if self.backtest is True:
                 # Running in backtest mode
+                
+                if self.validation_file is not None:
+                    # Extract instrument-specific trade history as trade summary and trade period
+                    formatted_instrument = instrument[:3] + "/" + instrument[-3:]
+                    raw_livetrade_summary = livetrade_history[livetrade_history.Instrument == formatted_instrument] # FOR OANDA
+                    from_date           = pd.to_datetime(raw_livetrade_summary.Date.values)[0]
+                    to_date             = pd.to_datetime(raw_livetrade_summary.Date.values)[-1]
+                    
+                    # Modify from date to improve backtest
+                    from_date = from_date - period*timedelta(seconds = self.granularity_to_seconds(interval))
+                    
+                    # Modify starting balance
+                    broker.portfolio_balance = raw_livetrade_summary.Balance.values[np.isfinite(raw_livetrade_summary.Balance.values)][0]
+                    
+                    
                 starting_balance    = broker.get_balance()
                 NAV     = []
                 balance = []
@@ -185,6 +229,7 @@ class AutoTrader():
                         # Check if historical data already exists
                         historical_data_name = 'hist_{0}{1}.csv'.format(interval, instrument)
                         historical_quote_data_name = 'hist_{0}{1}_quote.csv'.format(interval, instrument)
+                        data_dir_path = os.path.join(home_dir, 'price_data')
                         historical_data_file_path = os.path.join(home_dir, 
                                                                  'price_data',
                                                                  historical_data_name)
@@ -205,6 +250,11 @@ class AutoTrader():
                                                                                           to_date)
                             data, quote_data    = utils.check_dataframes(data, quote_data)
                             
+                            # Check if price_data folder exists
+                            if not os.path.exists(data_dir_path):
+                                # If price data directory doesn't exist, make it
+                                os.makedirs(data_dir_path)
+                                
                             # Save data in file/s
                             data.to_csv(historical_data_file_path)
                             quote_data.to_csv(historical_quote_data_file_path)
@@ -290,13 +340,7 @@ class AutoTrader():
                     if data_ts != last_candle_closed.timestamp():
                         print("  Could not retrieve updated data. Manually adjusting.")
                         data = broker.update_data(instrument, interval, data)
-            
-            
-            # Adjust plot output to avoid excessive data visualisation
-            if self.show_plot is True:
-                if len(data) < 75000:
-                    params['view_window']   = len(data)
-                    params['show_fig']      = True
+                        
                 
             # Instantiate strategy for current instrument
             my_strat    = strategy(params, data, instrument)
@@ -320,106 +364,133 @@ class AutoTrader():
                 candle              = data.iloc[i]
                 
                 # Run strategy to get signals
+                # TODO - move this into order_processing method
                 signal_dict = my_strat.generate_signal(i, open_positions)
-                signal      = signal_dict["direction"]
+                if 0 not in signal_dict:
+                    # Single order signal, nest in dictionary to allow iteration
+                    run_iteration = signal_dict["direction"]
+                    signal_dict = {1: signal_dict}
+                    
+                # Begin iteration over signal_dict to extract each order
+                for order in signal_dict:
+                    order_signal_dict = signal_dict[order]
                 
-                if signal != 0:
-                    # Signal detected
-                    if int(self.verbosity) > 0 and self.backtest is False:
-                        print("  Signal detected at {}: {}@{}".format(data.index[i],
-                                                                      signal,
-                                                                      data.Close[i]
-                                                                      ))
+                    signal      = order_signal_dict["direction"]
                     
-                    
-                    # Entry signal detected
-                    if self.backtest is True:
-                        datetime_stamp  = data.index[i]
-                        price_data      = broker.get_price(instrument, data, 
-                                                           quote_data, i)
-                    else:
-                        datetime_stamp  = datetime.now().strftime("%H:%M:%S")
-                        price_data      = broker.get_price(instrument)
-                    
-                    if signal < 0:
-                        price       = price_data['bid']
-                        HCF         = price_data['negativeHCF']
-                    else:
-                        price       = price_data['ask']
-                        HCF         = price_data['positiveHCF']
-                    
-                    # Get exit levels
-                    stop_price  = signal_dict["stop_loss"]
-                    stop_type   = signal_dict["stop_type"]
-                    take_price  = signal_dict["take_profit"]
-                    
-                    # Calculate risked amount
-                    amount_risked   = broker.get_balance() * risk_pc / 100
-                    
-                    # Calculate size
-                    if sizing == 'risk':
-                        size            = utils.get_size(instrument,
-                                                         amount_risked, 
-                                                         price, 
-                                                         stop_price, 
-                                                         HCF
-                                                         )
-                    else:
-                        size = sizing
-                    
-                    # Construct order dict
-                    order_details = {"order_time":      datetime_stamp,
-                                     "strategy":        my_strat.name,
-                                     "order_type":      signal_dict["order_type"],
-                                     "instrument":      instrument,
-                                     "size":            signal*size,
-                                     "price":           price,
-                                     "stop_loss":       stop_price,
-                                     "take_profit":     take_price,
-                                     "HCF":             HCF,
-                                     "stop_type":       stop_type,
-                                     "granularity":     interval
-                                     }
-                    
-                    # Place order
-                    if self.backtest is True:
-                        broker.place_order(order_details)
-                            
-                    else:
-                        # Running in live-trade mode
-                        if self.scan is not None:
-                            scan_hit = {"size"  : size,
-                                        "entry" : price,
-                                        "stop"  : stop_price,
-                                        "take"  : take_price,
-                                        "signal": signal
-                                        }
-                            scan_results[instrument] = scan_hit
+                    # TODO: this if statement is almost redundant now, consider deleting
+                    if signal != 0:
+                        # Signal detected
+                        if int(self.verbosity) > 0 and self.backtest is False:
+                            print("  Signal detected at {}: {}@{}".format(data.index[i],
+                                                                          signal,
+                                                                          data.Close[i]
+                                                                          ))
+                        
+                        # Entry signal detected, get price data
+                        if self.backtest is True:
+                            datetime_stamp  = data.index[i]
+                            price_data      = broker.get_price(instrument, data, 
+                                                               quote_data, i)
                         else:
-                            output = broker.place_order(order_details)
-                            
-                            # if int(self.verbosity) > 0:
-                            #     print("Order message:")
-                            #     print(output['Message'])
-                            
-                            # Send email
-                            if int(self.notify) > 0:
-                                utils.write_to_order_summary(order_details, 
-                                                             order_summary_fp)
+                            datetime_stamp  = datetime.now().strftime("%H:%M:%S")
+                            price_data      = broker.get_price(instrument)
+                        
+                        
+                        if signal < 0:
+                            order_price = price_data['bid']
+                            HCF         = price_data['negativeHCF']
+                        else:
+                            order_price = price_data['ask']
+                            HCF         = price_data['positiveHCF']
+                        
+                        # Define 'working_price' to calculate size and TP
+                        if order_signal_dict["order_type"] == 'limit' or order_signal_dict["order_type"] == 'stop-limit':
+                            working_price = order_signal_dict["order_limit_price"]
+                        else:
+                            working_price = order_price
+                        
+                        # Calculate exit levels
+                        stop_price = order_signal_dict['stop_loss'] if 'stop_loss' in order_signal_dict else None
+                        stop_distance = order_signal_dict['stop_distance'] if 'stop_distance' in order_signal_dict else None
+                        stop_type = order_signal_dict['stop_type'] if 'stop_type' in order_signal_dict else None
+                        
+                        if 'take_profit' not in order_signal_dict and \
+                            'take_distance' in order_signal_dict and \
+                            order_signal_dict['take_distance'] is not None:
+                            # Take profit distance specified
+                            pip_value   = utils.get_pip_ratio(instrument)
+                            take_profit = working_price + np.sign(signal)*order_signal_dict['take_distance']*pip_value
+                        else:
+                            # Take profit price specified, or no take profit specified at all
+                            take_profit = order_signal_dict["take_profit"] if 'take_profit' in order_signal_dict else None
+                        
+                        # Calculate risked amount
+                        amount_risked = broker.get_balance() * risk_pc / 100
+                        
+                        # Calculate size
+                        if 'size' in order_signal_dict:
+                            size = order_signal_dict['size']
+                        else:
+                            if sizing == 'risk':
+                                size            = utils.get_size(instrument,
+                                                                 amount_risked, 
+                                                                 working_price, 
+                                                                 stop_price, 
+                                                                 HCF,
+                                                                 stop_distance)
+                            else:
+                                size = sizing
+                        
+                        # Construct order dict by building on signal_dict
+                        order_details                   = order_signal_dict
+                        order_details["order_time"]     = datetime_stamp
+                        order_details["strategy"]       = my_strat.name
+                        order_details["instrument"]     = instrument
+                        order_details["size"]           = signal*size
+                        order_details["order_price"]    = order_price
+                        order_details["HCF"]            = HCF
+                        order_details["granularity"]    = interval
+                        order_details["stop_distance"]  = stop_distance
+                        order_details["stop_loss"]      = stop_price
+                        order_details["take_profit"]    = take_profit
+                        order_details["stop_type"]      = stop_type
+                        order_details["related_orders"] = order_signal_dict['related_orders'] if 'related_orders' in order_signal_dict else None
+                        
+                        # Place order
+                        if self.backtest is True:
+                            broker.place_order(order_details)
                                 
-                                if int(self.notify) > 1 and \
-                                    mailing_list is not None and \
-                                    host_email is not None:
-                                    emailing.send_order(order_details,
-                                                        output,
-                                                        mailing_list,
-                                                        host_email)
-            
-                else:
-                    # No signal detected
-                    if int(self.verbosity) > 0 and self.backtest is False:
-                        print("  No signal detected.\n")
-                   
+                        else:
+                            # Running in live-trade mode
+                            if self.scan is not None:
+                                scan_hit = {"size"  : size,
+                                            "entry" : order_price,
+                                            "stop"  : stop_price,
+                                            "take"  : take_profit,
+                                            "signal": signal
+                                            }
+                                scan_results[instrument] = scan_hit
+                            else:
+                                output = broker.place_order(order_details)
+                                
+                                # Send email
+                                if int(self.notify) > 0:
+                                    utils.write_to_order_summary(order_details, 
+                                                                 order_summary_fp)
+                                    
+                                    if int(self.notify) > 1 and \
+                                        mailing_list is not None and \
+                                        host_email is not None:
+                                        emailing.send_order(order_details,
+                                                            output,
+                                                            mailing_list,
+                                                            host_email)
+                
+                    else:
+                        # No signal detected
+                        if int(self.verbosity) > 0 and self.backtest is False:
+                            print("  No signal detected.\n")
+                    # End signal_dict iteration
                 
                 if self.backtest is True:
                     broker.update_positions(candle)
@@ -427,9 +498,19 @@ class AutoTrader():
                     balance.append(broker.portfolio_balance)
                     
                     
-            # Iteration complete
+            # End data iteration 
             if self.backtest is True:
                 trade_summary = utils.trade_summary(instrument, broker.closed_positions)
+                open_trade_summary = utils.open_order_summary(instrument, broker.open_positions)
+                cancelled_summary = utils.cancelled_order_summary(instrument, broker.cancelled_orders)
+                
+                if self.validation_file is not None:
+                    livetrade_summary = broker_utils.trade_summary(raw_livetrade_summary,
+                                                                            data,
+                                                                            interval)
+                    final_balance_diff  = NAV[-1] - livetrade_summary.Balance.values[-1]
+                    filled_live_orders  = livetrade_summary[livetrade_summary.Transaction == 'ORDER_FILL']
+                    no_live_trades      = len(filled_live_orders)
             
             if self.show_plot is True:
                 # Plot results
@@ -439,23 +520,29 @@ class AutoTrader():
                               "({} candles).".format(len(data)),
                               "Check saved figure.")
                     else:
-                        # pf_df = utils.reconstruct_portfolio(starting_balance, 
-                        #                               trade_summary, 
-                        #                               data.index)
-                        # my_strat.create_backtest_chart(instrument, interval, 
-                        #                                trade_summary, pf_df)
                         backtest_dict = {}
                         backtest_dict['data']           = data
-                        backtest_dict['balance']        = balance
                         backtest_dict['NAV']            = NAV
                         backtest_dict['trade_summary']  = trade_summary
-                        backtest_dict['indicators']     = my_strat.indicators
+                        backtest_dict['indicators']     = my_strat.indicators if hasattr(my_strat, 'indicators') else None
                         backtest_dict['pair']           = instrument
                         backtest_dict['interval']       = interval
-                        plot_backtest(backtest_dict)
-                else:
-                    my_strat.create_price_chart(instrument, interval)
-            
+                        backtest_dict['open_trades']    = open_trade_summary
+                        backtest_dict['cancelled_trades'] = cancelled_summary
+                        ap = autoplot.AutoPlot()
+                        ap.data = data
+                        
+                        if self.validation_file is None:
+                            ap.plot_backtest(backtest_dict)
+                        else:
+                            ap.plot_validation_balance = self.plot_validation_balance
+                            ap.ohlc_height = 350
+                            ap.validate_backtest(livetrade_summary, 
+                                                 backtest_dict,
+                                                 cancelled_summary,
+                                                 instrument, 
+                                                 interval)
+                            
             
             ''' -------------------------------------------------------------- '''
             '''              Construct backtest results dictionary             '''
@@ -554,8 +641,9 @@ class AutoTrader():
                 print("-------------------------------------------")
                 print("Strategy: {}".format(my_strat.name))
                 print("Timeframe:               {}".format(interval))
-                print("Risk to reward ratio:    {}".format(params['RR']))
-                print("Profitable win rate:     {}%".format(round(100/(1+params['RR']), 1)))
+                if params is not None and 'RR' in params:
+                    print("Risk to reward ratio:    {}".format(params['RR']))
+                    print("Profitable win rate:     {}%".format(round(100/(1+params['RR']), 1)))
                 if no_trades > 0:
                     print("Backtest win rate:       {}%".format(round(win_rate, 1)))
                     
@@ -604,6 +692,13 @@ class AutoTrader():
                 else:
                     print("There were no short trades.")
                 
+                if self.validation_file is not None:
+                    print("\n            Backtest Validation")
+                    print("-------------------------------------------")
+                    print("Difference between final portfolio balance between")
+                    print("live-trade account and backtest is ${}.".format(round(final_balance_diff, 2)))
+                    print("Number of live trades: {} trades.".format(no_live_trades))
+                
                 
             if self.log is True and self.backtest is True:
                 logger.write_backtest_log(instrument, config, trade_summary)
@@ -667,7 +762,25 @@ class AutoTrader():
         with open(file_path, "r") as f:
             return yaml.safe_load(f)
     
-    
+    def granularity_to_seconds(self, granularity):
+        '''Converts the interval to time in seconds'''
+        letter = granularity[0]
+        
+        if len(granularity) > 1:
+            number = float(granularity[1:])
+        else:
+            number = 1
+        
+        conversions = {'S': 1,
+                       'M': 60,
+                       'H': 60*60,
+                       'D': 60*60*24
+                       }
+        
+        my_int = conversions[letter] * number
+        
+        return my_int
+
     def print_usage(self):
         """ Print usage options. """
         banner = pyfiglet.figlet_format("AUTOTRADER")
