@@ -10,7 +10,7 @@ import pandas as pd
 
 
 def supertrend(data, period = 10, ATR_multiplier = 3.0):
-    '''  '''
+    ''' Based on the SuperTrend indicator by KivancOzbilgic on TradingView '''
     hl2             = (data.High.values + data.Low.values) / 2
     true_range      = ta.TRANGE(data.High.values, data.Low.values, data.Close.values)
     atr             = ta.SMA(true_range, timeperiod=period)
@@ -67,11 +67,15 @@ def supertrend(data, period = 10, ATR_multiplier = 3.0):
         
         trend_list.append(trend)
     
-    up_trend = pd.DataFrame({'uptrend': up_list}, index = up_times)
-    dn_trend = pd.DataFrame({'downtrend': dn_list}, index = dn_times)
+    # up_trend = pd.DataFrame({'uptrend': up_list}, index = up_times)
+    # dn_trend = pd.DataFrame({'downtrend': dn_list}, index = dn_times)
     
+    supertrend_df = pd.DataFrame({'uptrend': up_list,
+                                  'downtrend': dn_list,
+                                  'trend': trend_list}, 
+                                 index = up_times)
     
-    return up_trend, dn_trend, trend_list
+    return supertrend_df
 
 
 def stoch_rsi(data, K_period=3, D_period=3, RSI_length=14, Stochastic_length=14):
@@ -143,25 +147,28 @@ def crossover(list_1, list_2):
 
 
 def cross_values(a, b, ab_crossover):
-        cross_point_list = [0]
-        last_cross_point = 0
-        for i in range(1, len(ab_crossover)):
-            if ab_crossover[i] != 0:
-                i0 = 0
-                m_a = a[i] - a[i-1]
-                m_b = b[i] - b[i-1]
-                ix = (b[i-1] - a[i-1])/(m_a-m_b) + i0
-                
-                cross_point = m_a*(ix - i0) + a[i-1]
-                
-                last_cross_point = cross_point
-                
-            else:
-                cross_point = last_cross_point #0
+    cross_point_list = [0]
+    last_cross_point = 0
+    for i in range(1, len(ab_crossover)):
+        if ab_crossover[i] != 0:
+            i0 = 0
+            m_a = a[i] - a[i-1]
+            m_b = b[i] - b[i-1]
+            ix = (b[i-1] - a[i-1])/(m_a-m_b) + i0
             
-            cross_point_list.append(cross_point)
+            cross_point = m_a*(ix - i0) + a[i-1]
             
-        return cross_point_list
+            last_cross_point = cross_point
+            
+        else:
+            cross_point = last_cross_point #0
+        
+        cross_point_list.append(cross_point)
+    
+    # Replace nans with 0
+    cross_point_list = [0 if x!=x  else x for x in cross_point_list]
+    
+    return cross_point_list
 
 
 def candles_between_crosses(cross_list):
@@ -480,6 +487,60 @@ def ha_candle_run(ha_data):
         
     return green_run, red_run
 
+def build_grid(grid_origin, grid_space, grid_levels, order_direction, order_type='stop-limit',
+               pip_value=0.0001, take_distance=None, stop_distance=None, stop_type='limit'):
+    '''
+    grid_origin: origin of grid, specified as a price
+    grid_space: spacing between grid levels, specified as pip distance
+    grid_levels: number of grid levels either side of origin
+    order_direction: the direction of each grid level order (1 for long, -1 for short)
+    order_type: the order type of each grid level order
+    '''
+    # TODO - generalise to accept grid space in price units as well
+    # TODO - could add a limit price buffer, then use it to move the limit price
+    # slightly away from the stop price
+    
+    # Calculate grid spacing in price units
+    grid_price_space = grid_space*pip_value
+    
+    # Generate order_limit_price list 
+    order_limit_prices = np.linspace(grid_origin - grid_levels*grid_price_space, 
+                                     grid_origin + grid_levels*grid_price_space, 
+                                     2*grid_levels + 1)
+    
+    # Construct nominal order
+    nominal_order = {}
+    nominal_order["order_type"]         = order_type
+    nominal_order["direction"]          = order_direction
+    nominal_order["stop_distance"]      = stop_distance
+    nominal_order["stop_type"]          = stop_type
+    nominal_order["take_distance"]      = take_distance
+    
+    # Build grid
+    grid = {}
+
+    for order, limit_price in enumerate(order_limit_prices):
+        grid[order] = nominal_order.copy()
+        grid[order]["order_stop_price"]  = order_limit_prices[order]
+        grid[order]["order_limit_price"] = order_limit_prices[order]
+        
+    
+    return grid
+
+def merge_grid_orders(grid_1, grid_2):
+    '''
+    Merges grid dictionaries into one and re-labels order numbers so each
+    order number is unique.
+    '''
+    # TODO - use **args/**kwargs to generalise how many grids are inputted
+    order_offset = len(grid_1)
+    grid = grid_1.copy()
+    
+    for order_no in grid_2:
+        grid[order_no + order_offset] = grid_2[order_no]
+    
+    return grid
+    
 
 # def TDI(data):
 #     rsiPeriod = input(11, minval = 1, title = "RSI Period")
